/*
 * @brief This header file is part of Ding RTC engine sdk for iOS/Mac system
 *
 * @copyright Copyright (c) 2021, Alibaba Inc.
 * All rights reserved
 */

#import <DingRTC/DingRtcEngineTypes.h>

/**
 * @defgroup OC_DingRtcEngine 全部接口
 * DingRtc 接口
 */

/** @defgroup OC_DingRtcEngineBase 基础接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 基础接口
 */

/** @defgroup OC_DingRtcEngineChannel 频道相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 频道相关接口
 */

/** @defgroup OC_DingRtcEnginePub 发布相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 发布相关接口
 */

/** @defgroup OC_DingRtcEngineSub 订阅相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 订阅相关接口
 */

/** @defgroup OC_DingRtcEngineVideo 视频相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 视频相关接口
 */

/** @defgroup OC_DingRtcEngineScreenShare 共享视频相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 共享视频相关接口
 */

/** @defgroup OC_DingRtcEngineAudio 音频相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 音频相关接口
 */

/** @defgroup OC_DingRtcEngineMediaEngine 媒体引擎相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 媒体引擎相关接口
 */

/** @defgroup OC_DingRtcEngineLiveBypass 直播旁路相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 直播旁路相关接口
 */

/** @defgroup OC_DingRtcEngineChannelRelay 跨频道连麦相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 跨频道连麦相关接口
 */

/** @defgroup OC_DingRtcEnginePreview 预览相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 预览相关接口
 */

/** @defgroup OC_DingRtcEngineUserInfo 远端用户查询接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 远端用户查询接口
 */

/** @defgroup OC_DingRtcEngineOthers 其他接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 其他接口
 */

/**
* @defgroup OC_DingRtcCallback 回调接口
* DingRtc 关键类型定义
*/

#pragma mark - 视频观察器
/**
 * @defgroup OC_DingRtcCallbackVideoFrameObserver 视频数据回调及监听
 * @ingroup OC_DingRtcCallback
 * DingRtc 视频数据回调及监听
 * @{
 */
@protocol DingRtcVideoFrameDelegate <NSObject>

@required

/**
 * @ingroup OC_DingRtcCallbackVideoFrameObserver
 *
 * @since 3.0
 * @brief 视频格式采集偏好。
 */
- (DingRtcVideoFormat)getVideoFormatPreference;

@optional

/**
 * @ingroup OC_DingRtcCallbackVideoFrameObserver
 *
 * @since 3.0
 * @brief 本地采集视频数据。
 */
- (BOOL)onCaptureVideoFrame:(DingRtcVideoFrame * _Nonnull)frame;

/**
 * @ingroup OC_DingRtcCallbackVideoFrameObserver
 *
 * @since 3.1
 * @brief 远端解码后视频数据。
 */
- (BOOL)onRemoteVideoFrame:(DingRtcVideoFrame * _Nonnull)frame userId:(NSString *_Nonnull)userId videoTrack:(DingRtcVideoTrack)videoTrack;

/**
 * @ingroup OC_DingRtcCallbackVideoFrameObserver
 *
 * @since 3.1
 * @brief 本地编码前视频数据。
 */
- (BOOL)onPreEncodeVideoFrame:(DingRtcVideoFrame * _Nonnull)frame videoTrack:(DingRtcVideoTrack)videoTrack;

@end
/**
 * @}
*/

#pragma mark - 音频观察器
/**
 * @defgroup OC_DingRtcCallbackAudioFrameObserver 音频数据回调及监听
 * @ingroup OC_DingRtcCallback
 * DingRtc 音频数据回调及监听
 * @{
 */
@protocol DingRtcAudioFrameDelegate <NSObject>

@optional
/**
 * @brief 本地订阅音频数据回调。
 * @details 远端所有用户混音后待播放的音频数据。
 * @param frame 音频数据。
 * @since 3.0
*/
- (void)onPlaybackAudioFrame:(DingRtcAudioDataSample * _Nonnull)frame;

/**
 * @brief 本地采集音频数据回调。
 * @param frame 音频数据。
 * @since 3.0
 */
- (void)onCapturedAudioFrame:(DingRtcAudioDataSample * _Nonnull)frame;

/**
 * @brief 经过3A处理后的数据回调。
 * @param frame 音频数据。
 * @since 3.0
*/
- (void)onProcessCapturedAudioFrame:(DingRtcAudioDataSample * _Nonnull)frame;

/**
 * @brief 本地推流音频数据回调。
 * @param frame 音频数据。
 * @since 3.0
 */
- (void)onPublishAudioFrame:(DingRtcAudioDataSample * _Nonnull)frame;

@end
/**
 * @}
*/

#pragma mark - 回调
/**
 * @defgroup OC_DingRtcCallbackEventListener 回调及监听
 * @ingroup OC_DingRtcCallback
 * DingRtc 回调及监听
 * @{
 */
@protocol DingRtcEngineDelegate <NSObject>
@optional

/**
 * @since 3.0
 * @brief 加入频道回调。
 * @details 当应用调用 {@link DingRtcEngine::joinChannel:name:onResultWithUserId:} 方法时，该回调表示成功/失败加入频道，并且返回频道加入的相关信息以及加入频道耗时。
 * @param result 加入频道结果
 *          - 0：成功。
 *          - 非0：失败。
 *   - {@link DingRtcErrJoinBadAppId} ：AppId不存在，请检查入会参数authinfo中appId是否正确。
 *   - {@link DingRtcErrJoinInvalidAppId} ：AppId已失效，请检查入会参数authinfo中appId是否正确。
 *   - {@link DingRtcErrJoinBadChannel} ：频道不存在，请检查入会参数authinfo中channelId是否正确。
 *   - {@link DingRtcErrJoinInvalidChannel} ：频道已失效，请检查入会参数authinfo中channelId是否正确。
 *   - {@link DingRtcErrJoinBadToken} ：token不存在，请检查入会参数authinfo生成算法是否与官网文档一致。
 *   - {@link DingRtcErrJoinTimeout} ：加入频道超时，请检查当前网络是否为弱网或限速。
 *   - {@link DingRtcErrJoinBadParam} ：参数错误，请检查入会参数authinfo生成是否存在非法字符，或与官网文档是否一致。
 *
 * @param channel 加入频道名。
 * @param userId   用户ID。
 * @param elapsed 加入频道耗时。
 * @note 此回调等同于joinChannel接口的block参数，二者择一处理即可。
 */
- (void)onJoinChannelResult:(int)result channel:(NSString *_Nonnull)channel userId:(NSString *_Nonnull)userId elapsed:(int)elapsed;

/**
 * @since 3.0
 * @brief 离开频道回调。
 * @details 调用 {@link leaveChannel} 方法后，该回调表示成功/失败离开频道，回调将会返回离会的result和该频道的基本信息。
 * @param result 离开频道结果，成功返回0，失败返回错误码。
 * @param stats 本次频道内会话的数据统计信息。
 * @note 调用 {@link DingRtcEngine::leaveChannel} 接口后返回，如果 {@link DingRtcEngine::leaveChannel} 后直接调用 {@link DingRtcEngine::destroy} 将不会收到此回调。
 */
- (void)onLeaveChannelResult:(int)result stats:(DingRtcStats *_Nonnull)stats;


/**
 * @since 3.0
 * @brief 频道剩余时长回调。
 * @param remainingTimeInSec 频道剩余时长，单位秒。
 */
- (void)onChannelRemainingTimeNotify:(int)remainingTimeInSec;

/**
 * @since 3.0
 * @brief 音频推流变更回调。
 * @param oldState 之前的推流状态，详见{@link DingRtcPublishState}。
 * @param newState 当前的推流状态，详见{@link DingRtcPublishState}。
 * @param elapseSinceLastState 状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
 */
- (void)onAudioPublishStateChanged:(DingRtcPublishState)oldState newState:(DingRtcPublishState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 视频推流变更回调。
 * @param oldState 之前的推流状态，详见{@link DingRtcPublishState}。
 * @param newState 当前的推流状态，详见{@link DingRtcPublishState}。
 * @param elapseSinceLastState 状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
*/
- (void)onVideoPublishStateChanged:(DingRtcPublishState)oldState newState:(DingRtcPublishState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 屏幕分享推流变更回调。
 * @param oldState 之前的推流状态，详见{@link DingRtcPublishState}。
 * @param newState 当前的推流状态，详见{@link DingRtcPublishState}。
 * @param elapseSinceLastState 状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
*/
- (void)onScreenSharePublishStateChanged:(DingRtcPublishState)oldState newState:(DingRtcPublishState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 远端用户（通信模式）加入频道回调。
 * @details 该回调在以下场景会被触发。
 * - 通信模式：远端用户加入频道会触发该回调，如果当前用户在加入频道前已有其他用户在频道中，当前用户加入频道后也会收到已加入频道用户的回调。
 *
 * @param uid 用户ID 从App server分配的唯一标示符。
 * @param elapsed 用户加入频道时的耗时。
 */
- (void)onRemoteUserOnLineNotify:(NSString *_Nonnull)uid elapsed:(int)elapsed;

/**
 * @since 3.0
 * @brief 远端用户（通信模式）离开频道回调。
 * @details 该回调在以下场景会被触发。
 * - 通信模式：远端用户离开频道会触发该回调。
 * - 通信模式情况下，当长时间收不到远端用户数据，超时掉线时，会触发该回调。
 *
 * @param uid 用户ID 从App server分配的唯一标示符。
 * @param reason 用户离线的原因，详见 {@link DingRtcUserOfflineReason}。
 */
- (void)onRemoteUserOffLineNotify:(NSString *_Nonnull)uid offlineReason:(DingRtcUserOfflineReason)reason;

/**
 * @since 3.0
 * @brief 远端用户的音视频流发生变化回调。
 * @details 该回调在以下场景会被触发。
 * - 当远端用户从未推流变更为推流（包括音频和视频）。
 * - 当远端用户从已推流变更为未推流（包括音频和视频）。
 *
 * @param uid userId，从App server分配的唯一标示符。
 * @param audioTrack 音频流类型，详见 {@link DingRtcAudioTrack}。
 * @param videoTrack 视频流类型，详见 {@link DingRtcVideoTrack}。
 */
- (void)onRemoteTrackAvailableNotify:(NSString *_Nonnull)uid audioTrack:(DingRtcAudioTrack)audioTrack videoTrack:(DingRtcVideoTrack)videoTrack;

/**
 * @since 3.0
 * @brief 音频流订阅情况变更回调。
 * @param uid 用户ID，从App server分配的唯一标示符。
 * @param oldState 之前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param newState 当前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
 */
- (void)onAudioSubscribeStateChanged:(NSString *_Nonnull)uid oldState:(DingRtcSubscribeState)oldState newState:(DingRtcSubscribeState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 相机流订阅情况变更回调。
 * @param uid 用户ID，从App server分配的唯一标示符。
 * @param oldState 之前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param newState 当前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
 */
- (void)onVideoSubscribeStateChanged:(NSString *_Nonnull)uid oldState:(DingRtcSubscribeState)oldState newState:(DingRtcSubscribeState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 屏幕分享流订阅情况变更回调。
 * @param uid 用户ID，从App server分配的唯一标示符。
 * @param oldState 之前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param newState 当前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
 */
- (void)onScreenShareSubscribeStateChanged:(NSString *_Nonnull)uid oldState:(DingRtcSubscribeState)oldState newState:(DingRtcSubscribeState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 相机流大小规格订阅情况变更回调。
 * @param uid 用户ID，从App server分配的唯一标示符。
 * @param oldStreamType 之前的订阅的规格类型，详见 {@link DingRtcVideoStreamType}。
 * @param newStreamType 当前的订阅的规格类型，详见 {@link DingRtcVideoStreamType}。
 * @param elapseSinceLastState 大小流类型变更时间间隔（毫秒）。
 * @param channel 当前频道id。
 */
- (void)onSubscribeStreamTypeChanged:(NSString *_Nonnull)uid oldStreamType:(DingRtcVideoStreamType)oldStreamType newStreamType:(DingRtcVideoStreamType)newStreamType elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 网络质量变化时发出的消息。
 * @param uid 网络质量发生变化的uid。
 * @param upQuality  上行网络质量。
 * @param downQuality  下行网络质量。
 * @note 当网络质量发生变化时触发，uid为""时代表self的网络质量变化。
 */
- (void)onNetworkQualityChanged:(NSString *_Nonnull)uid
               upNetworkQuality:(DingRtcNetworkQuality)upQuality
             downNetworkQuality:(DingRtcNetworkQuality)downQuality;

/**
 * @since 3.0
 * @brief 网络连接状态改变的回调。
 * @param status 当前状态值, 对应值参考枚举DingRtcConnectionStatus。
 * @param reason 引起状态变化的具体原因, 对应值参考枚举DingRtcConnectionStatusChangeReason。
*/
- (void)onConnectionStatusChanged:(DingRtcConnectionStatus)status reason:(DingRtcConnectionStatusChangeReason)reason;

/**
 * @since 3.0
 * @brief 被服务器踢出/会议结束频道的消息。
 * @param code 原因参考{@link DingRtcOnByeType}。
 */
- (void)onBye:(DingRtcOnByeType)code;

/**
 * @since 3.0
 * @brief 如果engine出现warning，通过这个回调通知app。
 * @param warn  Warning type。
 * @param message 警告描述。
 */
- (void)onOccurWarning:(int)warn message:(NSString *_Nonnull)message;

/**
 * @since 3.0
 * @brief 如果engine出现error，通过这个回调通知app。
 * @param error Error类型，参考{@link DingRtcErrorCode}。
 * @param message 错误描述。
 */
- (void)onOccurError:(int)error message:(NSString *_Nonnull)message;


/**
 * @since 3.0
 * @brief 音频首包发送回调。
 * @details 在首个音频数据包发送出去时触发此回调。
 * @param timeCost 发送耗时，从入会开始到音频首包发送出去的耗时。
 */
- (void)onFirstAudioPacketSentWithTimecost:(int)timeCost;

/**
 * @since 3.0
 * @brief 视频首包发送回调。
 * @param videoTrack 发送视频track，参考 {@link DingRtcVideoTrack}。
 * @param timeCost 耗时（毫秒）。
 * @note 该接口用于视频首包发送的回调。
*/
- (void)onFirstVideoPacketSentWithVideoTrack:(DingRtcVideoTrack)videoTrack
                                    timeCost:(int)timeCost;

/**
 * @since 3.0
 * @brief 远端用户音频首包接收回调。
 * @details 在接收到远端首个音频数据包时触发此回调。
 * @param uid 远端用户ID，从App server分配的唯一标识符。
 * @param timeCost 接收耗时，从入会开始到音频首包接收到的耗时。
 */
- (void)onFirstAudioPacketReceivedWithUid:(NSString *_Nonnull)uid
                                 timeCost:(int)timeCost;

/**
 * @since 3.0
 * @brief 远端用户视频首包接收回调。
 * @param uid User ID，从App server分配的唯一标示符。
 * @param videoTrack 接收视频track，参考 {@link DingRtcVideoTrack}。
 * @param timeCost 耗时（毫秒）。
 * @note 该接口用于视频首包接收的回调。
*/
- (void)onFirstVideoPacketReceivedWithUid:(NSString *_Nonnull)uid
                               videoTrack:(DingRtcVideoTrack)videoTrack
                                 timeCost:(int)timeCost;

/**
 * @since 3.0
 * @brief 远端用户视频首帧接收回调。
 * @param uid User ID，从App server分配的唯一标示符。
 * @param videoTrack 接收视频track，参考 {@link DingRtcVideoTrack}。
 * @param timeCost 耗时（毫秒）。
 * @note 该接口用于收到远端用户视频首帧的回调。
*/
- (void)onFirstVideoFrameReceivedWithUid:(NSString *_Nonnull)uid
                              videoTrack:(DingRtcVideoTrack)videoTrack
                                timeCost:(int)timeCost;

/**
 * @since 3.0
 * @brief 远端用户的第一帧视频帧显示时触发这个消息。
 * @param uid User ID，从App server分配的唯一标示符。
 * @param videoTrack 屏幕流或者相机流，参考 {@link DingRtcVideoTrack}。
 * @param width 视频宽度。
 * @param height 视频高度。
 * @param elapsed 本地用户加入频道直至该回调触发的延迟总耗时（毫秒）。
 * @note 该接口用于远端用户的第一帧视频帧显示时的回调。
 */
- (void)onFirstRemoteVideoFrameDrawn:(NSString *_Nonnull)uid videoTrack:(DingRtcVideoTrack)videoTrack width:(int)width height:(int)height elapsed:(int)elapsed;

/**
 * @since 3.0
 * @brief 预览开始显示第一帧视频帧时触发这个消息。
 * @param width 本地预览视频宽度。
 * @param height 本地预览视频高度。
 * @param elapsed 从本地用户加入频道直至该回调触发的延迟总耗时（毫秒）。
 * @note 该接口用于预览开始显示第一帧视频帧时的回调。
 */
- (void)onFirstLocalVideoFrameDrawn:(int)width height:(int)height elapsed:(int)elapsed;

/**
 * @since 3.2.1
 * @brief 视频渲染接收到远端视频分辨率有变化时触发此回调。
 * @param uid userId，从App server分配的唯一标示符。
 * @param videoTrack 接收视频track。
 * @param newWidth 变化后新的视频宽度
 * @param newHeight 变化后新的视频高度
 * @param oldWidth 变化前旧的视频宽度
 * @param oldHeight 变化前旧的视频高度
 * @note 视频大小可能和解码器输出的视频大小不相等，如果使用了超分一类的视频后处理。
 * @note 因为simulcast的存在，以及动态调整分辨率，横竖屏变化，视频大小和远端设置的最大推流分辨率不一定相同。
 */
- (void)onRenderRemoteVideoSizeChanged:(NSString *_Nonnull)uid videoTrack:(DingRtcVideoTrack)videoTrack newWidth:(int)newWidth newHeight:(int)newHeight oldWidth:(int)oldWidth oldHeight:(int)oldHeight;

/**
 * @since 3.2.1
 * @brief 视频渲染接收到本地视频分辨率有变化时触发此回调。
 * @param videoTrack 本地视频track。
 * @param newWidth 变化后新的视频宽度
 * @param newHeight 变化后新的视频高度
 * @param oldWidth 变化前旧的视频宽度
 * @param oldHeight 变化前旧的视频高度
 * @note 视频大小可能和摄像头采集到的视频大小不相等，比如在采集之后存在缩放等处理。
 */
- (void)onRenderLocalVideoSizeChanged:(DingRtcVideoTrack)videoTrack newWidth:(int)newWidth newHeight:(int)newHeight oldWidth:(int)oldWidth oldHeight:(int)oldHeight;

/**
 * @since 3.0
 * @brief 用户音量提示回调。
 * @param speakers 用户音量信息，表示回调用户音量信息数组，包含用户uid,语音状态以及音量，uid为"0"表示本地说话人。
 * @param speakerNumber 用户数量。
 */
- (void)onAudioVolumeIndication:(NSArray <DingRtcUserVolumeInfo *> *_Nullable)speakers speakerNumber:(int)speakerNumber;

/**
 * @since 3.0
 * @brief 远端用户静音/取消静音回调。
 * @param uid 远端用户ID。
 * @param isMute 
 * - 该用户是否静音
 * - YES：静音。
 * - NO：取消静音。
 */
- (void)onUserAudioMuted:(NSString *_Nonnull)uid audioMuted:(BOOL)isMute;

/**
 * @since 3.0
 * @brief 远端用户暂停/恢复视频流发送的通知。
 * @param uid 远端用户ID。
 * @param isMute
 *         -  YES：该用户暂停视频流的发送。
 *         -  NO：该用户恢复视频流的发送。
 * @param track 屏幕流或者相机流。
 */
- (void)onUserVideoMuted:(NSString *_Nonnull)uid videoMuted:(BOOL)isMute track:(DingRtcVideoTrack)track;

/**
 * @since 3.5
 * @brief 远端用户关闭相机流采集发送通知。
 * @param uid 远端用户ID。
 * @param enabled
 *        - YES：打开相机流采集。
 *        - NO：关闭相机流采集。
 */
- (void)onUserVideoEnabled:(NSString *_Nonnull)uid enabled:(BOOL)enabled;

/**
 * @since 3.5
 * @brief 远端用户应用退到后台
 * @param uid 远端用户ID
 */
- (void)onUserWillResignActive:(NSString *_Nonnull)uid;

/**
 * @since 3.5
 * @brief 远端用户应用返回前台
 * @param uid 远端用户ID
 */
- (void)onUserWillBecomeActive:(NSString *_Nonnull)uid;

/**
 * @since 3.0
 * @brief 质量报告事件，相对onRtcLocalAudioStats, onRtcRemoteVideoStats更加简练，适合在UI上展示。
 * @param quality 采集到的各项指标数据。
 */
- (void)onRtcQualityReport:(DingRtcQualityReport *_Nonnull)quality;

/**
 * @since 3.0
 * @brief 实时数据回调（2s触发一次）。
 * @param stats stats。
 */
- (void)onRtcStats:(DingRtcStats *_Nonnull)stats;

/**
 * @since 3.0
 * @brief 本地视频统计信息（2s触发一次）。
 * @param localVideoStats 本地视频统计信息。
 * @note SDK每两秒触发一次此统计信息回调。
 */
- (void)onRtcLocalVideoStats:(DingRtcLocalVideoStats *_Nonnull)localVideoStats;

/**
 * @since 3.0
 * @brief 远端视频统计信息（2s触发一次）。
 * @param remoteVideoStats 远端视频统计信息。
 */
- (void)onRtcRemoteVideoStats:(DingRtcRemoteVideoStats *_Nonnull)remoteVideoStats;

/**
 * @since 3.0
 * @brief 本地音频统计信息（2s触发一次）。
 * @param localAudioStats 本地视频统计信息。
 * @note SDK每两秒触发一次此统计信息回调。
 */
- (void)onRtcLocalAudioStats:(DingRtcLocalAudioStats *_Nonnull)localAudioStats;

/**
 * @since 3.0
 * @brief 远端音频统计信息（2s触发一次）。
 * @param remoteAudioStats 远端音频统计信息。
 */
- (void)onRtcRemoteAudioStats:(DingRtcRemoteAudioStats *_Nonnull)remoteAudioStats;

#if TARGET_OS_IPHONE
/**
 * @since 3.0
 * @remark iOS
 * @brief 音频路由发生变化回调（仅适用于 iOS 系统）。
 * @details 所谓“音频路由”，是指声音是从手机的扬声器还是从听筒中播放出来，音频路由变化也就是声音的播放位置发生了变化。
 * - 当音频路由为听筒时，声音比较小，只有将耳朵凑近才能听清楚，隐私性较好，适合用于接听电话。
 * - 当音频路由为扬声器时，声音比较大，不用将手机贴脸也能听清，因此可以实现“免提”的功能。
 *
 * @param routing 当前使用的音频路由，详情参考{@link DingRtcAudioRouteType}
 */
- (void)onAudioRouteChanged:(DingRtcAudioRouteType)routing;
#endif

#if TARGET_OS_MAC && !TARGET_OS_IPHONE

/**
 * @since 3.0
 * @remark mac
 * @brief 音频设备状态变更 (仅适用于 Mac 系统)。
 * @param deviceId  外接设备ID。
 * @param deviceType  外接设备类型。
 * @param deviceState 外接设备状态。
 */
- (void)onAudioDeviceStateChanged:(NSString *_Nonnull)deviceId deviceType:(DingRtcExternalDeviceType)deviceType deviceState:(DingRtcExternalDeviceState)deviceState;

/**
 * @since 3.4.1
 * @remark mac
 * @brief 摄像头设备状态变更 (仅适用于 Mac 系统)。
 * @param deviceId  外接设备ID。
 * @param deviceType  外接设备类型。
 * @param deviceState 外接设备状态。
 */
- (void)onVideoDeviceStateChanged:(NSString *_Nonnull)deviceId deviceType:(DingRtcExternalDeviceType)deviceType deviceState:(DingRtcExternalDeviceState)deviceState;

#endif

/**
 * @since 3.0
 * @brief 音频采集设备测试回调。
 * @param level 音频采集设备音量值。
 */
- (void)onAudioDeviceRecordLevel:(int)level;

/**
 * @since 3.0
 * @brief 音频播放设备测试回调。
 * @param level 音频采集设备音量值。
 */
- (void)onAudioDevicePlayoutLevel:(int)level;

/**
 * @since 3.0
 * @brief API 方法已执行回调。
 * @param error 当该方法调用失败时 SDK 返回的错误码。如果该方法调用成功，SDK 将返回 0。失败，非0。
 * @param api SDK 执行的 API。
 * @param result SDK 调用 API 的调用结果。
 */
- (void)onApiCalledExecuted:(int)error api:(NSString *_Nullable)api result:(NSString *_Nullable)result;

/**
 * @since 3.0
 * @brief 视频截图回调。
 * @param uid 用户uid，为nil或者为""代表本地用户。
 * @param videoTrack 截图视频track，参考 {@link DingRtcVideoTrack}。
 * @param filePath 截图的本地保存路径。
 * @param width 截图宽度。
 * @param height 截图高度。
 * @param image 截图数据。
 * @param success 截图结果。
 * @note 成功时image返回截图数据，失败返回nil。
 */
- (void)onSnapshotComplete:(NSString*_Nullable)uid videoTrack:(DingRtcVideoTrack)videoTrack filePath:(NSString* _Nullable)filePath width:(int)width height:(int)height success:(BOOL)success;

/**
 * 收到媒体扩展信息回调
 * @param uid 发送用户userId
 * @param data 媒体扩展信息
 * @note 当一端通过 {@link AliRtcEngine::sendMediaExtensionMsg:repeatCount:} 发送信息后，其他端通过该回调接收数据
 */
- (void)onMediaExtensionMsgReceived:(NSString *_Nonnull)uid message:(NSData *_Nonnull)data;

/**
 * @since 3.4
 * @brief 伴奏混音回调。
 * @param status 伴奏混音状态， 详细参考{@link DingRtcAudioMixingStatusConfig}。
 * @note 该接口用于伴奏混音状态监控。
 */
- (void)onAudioMixingStateChanged:(DingRtcAudioMixingStatusConfig *_Nonnull)status;

/**
 * @since 3.4
 * @brief 返回当前频道内所有的分组列表。
 * @param audioGroups 分组列表。
 * @note 该回调发生在加入频道的时候。App通过该回调可以获得当前频道内所有的分组。后续分组变化，通过回调 { @link onAudioGroupListUpdate }获得。
 */
- (void)onListAllAudioGroups:(NSArray <NSString *> *_Nonnull)audioGroups;

/**
 * @since 3.4
 * @brief 当调用{ @link joinAudioGroup:usrData: }加入某个 group，触发此回调。
 * @param result 0 表示成功，其他值表示失败。
 * @param errorMessage 如果 result 不为0，补充说明错误原因。
 * @param group 加入的分组。
 * @param members 该分组的成员用户列表。
 */
- (void)onAudioGroupJoinResult:(int)result errorMessage:(NSString *_Nullable)errorMessage group:(NSString *_Nonnull)group members:(NSArray <DingRtcAudioGroupMemberInfo *> *_Nonnull)members;

/**
 * @since 3.4
 * @brief 当调用{ @link leaveAudioGroup: }离开某个 group，触发此回调。
 * @param result 0 表示成功，其他值表示失败。
 * @param errorMessage 如果 result 不为0，补充说明错误原因。
 * @param group 离开的分组。
 */
- (void)onAudioGroupLeaveResult:(int)result errorMessage:(NSString *_Nullable)errorMessage group:(NSString *_Nonnull)group;

/**
 * @since 3.4
 * @brief 当调用{ @link dismissAudioGroup: }解散某个group，触发此回调。
 * @param result 0 表示成功，其他值表示失败。
 * @param errorMessage 如果 result 不为0，补充说明错误原因。
 * @param group 解散的分组。
 */
- (void)onAudioGroupDismissResult:(int)result errorMessage:(NSString *_Nullable)errorMessage group:(NSString *_Nonnull)group;

/**
 * @since 3.4
 * @brief 当调用{ @link mixAudioToGroup:mix: }向分组或者大厅混流，触发此回调。
 * @param result 0 表示成功，其他值表示失败。
 * @param reason 如果 result 不为 0，补充说明失败的原因。
 * @param group 目标分组。如果是{@link HallID}，则表示目标为大厅的音频，否则是普通分组的ID。非空。
 * @param mix YES 表示将本地音频流混进指定的 channel audio 或者分组音频中; NO表示取消。
 */
- (void)onAudioGroupMixResult:(int)result reason:(NSString *_Nullable)reason group:(NSString *_Nonnull)group mix:(BOOL)mix;

/**
 * @since 3.4
 * @brief 分组中成员变化，触发此回调。
 * @param updateOption 1表示新增了成员，-1表示离开了成员。
 * @param group 成员变化的分组。
 * @param members 变动的（新增的或者离开的）成员列表。
 */
- (void)onAudioGroupMemberUpdate:(int)updateOption group:(NSString *_Nonnull)group members:(NSArray <DingRtcAudioGroupMemberInfo *> *_Nonnull)members;

/**
 * @since 3.4
 * @brief 大厅中的成员列表变化，触发此回调。
 * @param hallMembers 大厅中的全量成员列表。
 * @note 大方会场景下，回调如果比较频繁，存在一定的 cpu 开销。
 */
- (void)onAudioGroupHallMembers:(NSArray <DingRtcAudioGroupMemberInfo *> *_Nonnull)hallMembers;

/**
 * @since 3.4
 * @brief 分组列表变化，触发此回调。
 * @param updateOption 1表示新增了分组，-1表示删除了分组。
 * @param group 该回调指示的分组。
 */
- (void)onAudioGroupListUpdate:(int)updateOption group:(NSString *_Nonnull)group;

@end
/** @}  */

#pragma mark - 接口主类

__attribute__((visibility("default"))) @interface DingRtcEngine : NSObject <DingRtcEngineDelegate>

#pragma mark - "Engine的生命周期，工作参数以及频道操作"
/**
 * @name Engine的生命周期，工作参数以及频道操作
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.0
 * @brief 创建并获取一个DingRtcEngine实例，多次调用返回同一实例，既单例。
 * @param delegate 用于接收DingRtc相关事件回调的代理。
 * @param extras 用于接收来自客户灰度下发的参数，通过JSON配置SDK的特别功能，可以是空字符串。
 */
+ (instancetype _Nonnull)sharedInstance:(id<DingRtcEngineDelegate>_Nullable)delegate extras:(NSString *_Nullable)extras;

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.0
 * @brief 销毁  {@link DingRtcEngine::sharedInstance:extras:} 创建的DingRtcEngine实例。
 * @details 调用该方法将会释放所有内部使用的资源,当完成音视频通信之后都建议调用该方法释放实例.调用该方法后，你将不能再使用 {@link DingRtcEngine} 的其他方法和任何回调。
 * @note 该方法为同步调用,需要等待内部资源释放之后才能执行其他方法,为避免主线程阻塞，建议开发者放在子线程调用该方法，但需要注意的是如需在销毁后再次创建 {@link DingRtcEngine} 实例，请开发者务必保证 destroy 方法执行结束后再创建实例。
 * @note 为避免死锁，不建议在任何SDK的回调中调用本方法。
 */
+ (void)destroy;

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.3.0
 * @brief 创建并获取一个DingRtcEngine实例，多次调用返回多个实例。
 * @param delegate 用于接收DingRtc相关事件回调的代理。
 * @param extras 用于接收来自客户灰度下发的参数，通过JSON配置SDK的特别功能，可以是空字符串。
 */
+ (instancetype _Nonnull)createInstance:(id<DingRtcEngineDelegate>_Nullable)delegate extras:(NSString *_Nullable)extras;

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.3.0
 * @brief 销毁  {@link DingRtcEngine::sharedInstance:extras:}  或者 {@link DingRtcEngine::createInstance:extras:}  创建的DingRtcEngine实例。
 * @param instance DingRtcEngine的实例。
 * @details 调用该方法将会释放所有内部使用的资源,当完成音视频通信之后都建议调用该方法释放实例.调用该方法后，你将不能再使用 {@link DingRtcEngine} 的其他方法和任何回调。
 * @note 该方法为同步调用,需要等待内部资源释放之后才能执行其他方法,为避免主线程阻塞，建议开发者放在子线程调用该方法，但需要注意的是如需在销毁后再次创建 {@link DingRtcEngine} 实例，请开发者务必保证 destroyInstance 方法执行结束后再创建实例。
 * @note 为避免死锁，不建议在任何SDK的回调中调用本方法。
 */
+ (void)destroyInstance:(DingRtcEngine *_Nonnull)instance;

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.0
 * @brief 查询SDK当前版本号。
 * @return 当前的SDK版本号，格式为字符串，例："3.0.0.0"。
 * @note 在任何时刻都可以调用。
 */
+ (NSString *_Nonnull)getSDKVersion;

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.0
 * @brief 设置SDK日志文件保存路径。
 * @param logDirPath 日志文件保存目录的绝对路径，该参数是一个目录路径，而不是文件的路径。
 * @return
 * - 0，成功；
 * - 非0，失败。其中有两种错误码，释义如下：
 *   - {@link DingRtcErrLogRootPathError}：路径已存在或因权限问题不可访问。
 *   - {@link DingRtcErrLogAlreadyInit}：log对象已经初始化过。
 *
 * @note 调用此接口，请在调用其他所有SDK接口前调用本接口，避免日志出现丢失。同时，App必须保证指定的目录已存在并且可以写入。
 * - 如果，目标路径不可写入或创建日志文件失败，则会使用默认日志路径。本次设置将无效。
 */
+ (int)setLogDirPath:(NSString *_Nullable)logDirPath;

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.0
 * @brief 设置日志等级。
 * @param logLevel Log级别，详见{@link DingRtcLogLevel}枚举类型。
 * @return
 * - 0：成功；
 * - 非0：失败。比如没有先调用setLogDirPath。
 *
 * @note 设置 SDK 的输出日志等级。默认值为 {@link DingRtcLogLevel::DingRtcLogLevelInfo} 。
 * - 调用此接口，请在仅次于SetLogDirPath接口调用之后再调用本接口，避免因日志等级切换造成的打印丢失。
 */
+ (int)setLogLevel:(DingRtcLogLevel)logLevel;

/**
 * @ingroup OC_DingRtcEngineOthers
 *
 * @since 3.0
 * @brief 根据错误码，获取错误码描述。
 * @param errCode 错误码，详见{@link DingRtcErrorCode}。
 * @return 错误码描述字符串。
 * @note 在任何时刻都可以调用。
 */
+ (NSString *_Nullable)getErrorDescription:(NSInteger)errCode;

/**
 * @ingroup OC_DingRtcEngineOthers
 *
 * @since 3.0
 * @brief 是否分发回调到主线程。
 * @param enabled 
 *          - YES：回调分发至主线程队列。
 *          - NO：回调不分发至主线程队列。
 * @note 回调默认分发至主线程队列（iOS Default:YES，Mac Default:NO）。
 * @return 0表示Success；非0表示Failure。
 */
- (int)enableDelegateMainQueue:(BOOL)enabled;

/**
 * @ingroup OC_DingRtcEngineOthers
 *
 * @since 3.0
 * @brief 指定回调线程队列。
 * @param queue 回调queue。
 * @note enableDelegateMainQueue为YES时，此接口设置无效，回调线程为主线程。
 * - enableDelegateMainQueue为NO时，可通过此接口指定回调线程，若不设置，则使用SDK的默认子线程。
 * @return 0表示Success；非0表示Failure。
 */
- (int)setDelegateQueue:(NSOperationQueue *_Nullable)queue;

/**
 * @ingroup OC_DingRtcEngineChannel
 *
 * @since 3.0
 * @brief 加入频道。
 * @details 该方法让用户加入频道，一个频道内可以加入多个用户进行互相通话。
 * @param authInfo   认证信息，从App Server获取。
 * @param userName   任意用于显示的用户名称。不是用户ID。
 * @param onResult   当joinChannel执行结束后调用这个回调。
 * @return
 * - 0：成功；
 * - 非0：失败。
 * @note
 *    - 正常情况一个Appid对应一个App，只有同一个AppId的应用可进行互相通话,不同AppId和channelID不同都不能互通。
 *    - 当入会成功后，会触发远端用户收到 {@link onRemoteUserOnLineNotify:elapsed:} 回调。
 *    - 如果已在会中想重新入会，需要先调用 {@link leaveChannel} 离会，且确保 {@link onLeaveChannelResult:stats:} 回调收到后，再次调用进行入会。
 */
- (int)joinChannel:(DingRtcAuthInfo *_Nonnull)authInfo name:(NSString *_Nullable)userName onResultWithUserId:(void(^_Nullable)(NSInteger errCode, NSString * _Nonnull channel, NSString * _Nonnull userId, NSInteger elapsed))onResult;

/**
 * @ingroup OC_DingRtcEngineChannel
 *
 * @since 3.0
 * @brief  离开频道。
 * @details 调用该方法代表离开频道，即挂断或者停止通话。
 * - 该方法为异步方法，调用后收到 {@link onLeaveChannelResult:stats:} 代表退出频道完成，同时会触发远端用户收到 {@link onRemoteUserOffLineNotify:offlineReason:}   回调。
 * - 如果未加入频道入会，{@link leaveChannel} 不会有任何影响。加入频道后，必须要调用该方法离开频道，否则无法加入其他频道。
 * @return
 * - 0：成功；
 * - 非0：失败。
 *   - 失败原因错误码见回调函数：{@link  onLeaveChannelResult:stats:}。
 * @note
 *  - 如果调用了 {@link leaveChannel} 后立刻调用 {@link DingRtcEngine::destroy} 方法，SDK将会无法触发 {@link  onLeaveChannelResult:stats:} 回调。
 *
 */
- (int)leaveChannel;

/** @} */

#pragma mark - "设置和操作本地的媒体"
/**
 * @name 设置和操作本地的媒体
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 设置本地预览显示视图。
 * @param viewConfig 本地视频显示属性, 详细见{@link DingRtcVideoCanvas}。
 * @param track  只支持{@link DingRtcVideoTrackCamera}和{@link DingRtcVideoTrackScreen}。
 * @return
* - 0 ：成功；
 * - < 0 ：失败。
 * @note 支持joinChannel之前和之后切换窗口。如果canvas或者{@link DingRtcVideoCanvas::view}为nil，则停止显示。
 *       如果需要重新设置{@link DingRtcVideoCanvas::renderMode}，请保持canvas中其他属性不变，仅修改
 *       {@link DingRtcVideoCanvas::renderMode}。
 *       如果需要重新设置{@link DingRtcVideoCanvas::mirrorMode}，请保持canvas中其他属性不变，仅修改
 *       {@link DingRtcVideoCanvas::mirrorMode}。
 */
- (int)setLocalViewConfig:(DingRtcVideoCanvas *_Nullable)viewConfig forTrack:(DingRtcVideoTrack)track;

/**
 * @ingroup OC_DingRtcEnginePreview
 *
 * @since 3.0
 * @brief 开启本地视频预览。
 * @return
 * - 0 ：成功；
 * - < 0 ：失败。
 * @note 如果没有调用{@link #setLocalViewConfig:forTrack:}设置显示的view，则无法预览。
 * 可以在{@link #joinChannel:name:onResult:}之前就开启预览会自动打开摄像头。
 */
- (int)startPreview;

/**
 * @ingroup OC_DingRtcEnginePreview
 *
 * @since 3.0
 * @brief 关闭本地视频预览。
 * @return
 * - 0 ：成功；
 * - < 0 ：失败。
 * @note {@link #leaveChannel}会自动停止本地预览；如果没有在推相机流，则会自动关闭摄像头。
 */
- (int)stopPreview;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 检查camera是否打开。
 * @return
 * - YES：表示摄像头已打开；
 * - NO：表示摄像头没有打开。
 * @note 该接口用于检查当前camera的状态。
 */
- (BOOL)isCameraOn;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 禁用或启用本地视频采集。
 * @param enabled
 * - YES ：启用本地视频采集。
 * - NO ：禁用本地视频采集。
 * @return
 * - 0 ：成功；
 * - < 0 ：失败。
 * @note 默认为开启状态。
 */
- (int)enableLocalVideo:(BOOL)enabled;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 暂停/恢复本地视频流的发送。
 * @param mute
 *         - YES表示暂停视频流的发送。
 *         -  NO表示恢复视频流的发送。
 * @param track  只支持{@link DingRtcVideoTrackCamera 和 DingRtcVideoTrackScreen}。
 * @return
 * - 0：表示Success；
 * - 非0：表示Failure。
 * @note 视频流暂停发送时，采集和编码模块仍然在工作。
*/
- (int)muteLocalCamera:(BOOL)mute forTrack:(DingRtcVideoTrack)track;

/**
 * @ingroup OC_DingRtcEnginePub
 *
 * @since 3.0
 * @brief 是否推送本地视频流。
 * @param enabled 
 * - 是否开启/关闭本地视频流推流。
 * - YES：开启视频流推送。
 * - NO：关闭视频流推送。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 *   - DingRtcErrInner：SDK内部状态错误，需检查是否创建SDK实例成功。
 *
 * @note
 * - SDK默认设置为推送视频流，加入频道前调用此接口设置推送视频流，将在加入频道成功时生效，加入频道成功后可以随时调用此接口开启/关闭视频流推送。
 * - 本地视频流内容默认为摄像头采集视频，如需自定义输入视频推送内容，可参考接口{@link DingRtcEngine::setExternalVideoSource:track:}使用外部视频输入替换摄像头采集。
 * - 本地视频推流结果发生变化时，SDK会触发{@link DingRtcEngineDelegate::onVideoPublishStateChanged:newState:elapseSinceLastState:channel:}回调通知视频推流最新状态。
*/
- (int)publishLocalVideoStream:(BOOL)enabled;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 设置相机流视频编码属性。
 * @details 该方法用于设置相机流视频编码属性对应的视频参数，如分辨率、帧率、码率、视频方向等 所有设置的参数都有相应的范围限制，如果设置的参数不在有效范围内，SDK会自动调节。
 * @param config 预定义的编码属性，详见{@link DingRtcVideoEncoderConfiguration}。
 * @note
 * - 该方法在入会前和入会后都可以调用，如果每次入会只需要设置一次相机流视频编码属性，建议在入会前调用。
*/
- (void)setVideoEncoderConfiguration:(DingRtcVideoEncoderConfiguration* _Nonnull)config;

/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @brief 设置屏幕流视频编码属性。
 * @details 该方法用于设置屏幕流视频编码属性对应的视频参数，如分辨率、帧率、码率、视频方向等 所有设置的参数都有相应的范围限制，如果设置的参数不在有效范围内，SDK会自动调节。
 * @param config 预定义的屏幕共享编码属性，详见{@link DingRtcScreenShareEncoderConfiguration}。
 * @note
 * - 该方法在入会前和入会后都可以调用，如果每次入会只需要设置一次屏幕流视频编码属性，建议在入会前调用。
*/
- (void)setScreenShareEncoderConfiguration:(DingRtcScreenShareEncoderConfiguration* _Nonnull)config;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 开启或关闭视频美颜。
 * @private
 * @param enable 是否开启。
 * @param options 美颜参数。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)enableBeautyFace:(BOOL)enable withOption:(DingRtcBeautyFaceOptions* _Nonnull)options;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.3
 * @brief 开启或关闭虚拟背景。
 * @private
 * @param enable 是否开启。
 * @param options 虚拟背景参数。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)enableVirtualBackground:(BOOL)enable withOption:(DingRtcVirtualBackgroundOptions *)options;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 设置视频暗光增强功能。
 * @private
 * @param options 视频暗光增强参数。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setVideoEnhance:(DingRtcVideoEnhanceOptions* _Nonnull)options;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 设置视频去噪场景功能。
 * @private
 * @param options 视频去噪参数。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setVideoDenoise:(DingRtcVideoDenoiseOptions* _Nonnull)options;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 设置音频去噪功能。
 * @param options 音频去噪参数。
 * @return
 * - 0：成功；
 * - 非0：失败。
 * @note 音乐场景下会自动关闭去噪功能。
 */
- (int)setAudioDenoise:(DingRtcAudioDenoiseOptions* _Nonnull)options;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 设置视频帧观察器。
 * @param delegate DingRtcVideoFrameDelegate对象。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setVideoFrameDelegate:(id<DingRtcVideoFrameDelegate> _Nullable)delegate;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.1
 * @brief 设置视频帧数据观察开关。
 * @private
 * @param enable 是否允许数据回调。
 * @param position 视频数据类型，详见 {@link DingRtcVideoObservePosition}。
 * @return 0：sucess。
 */
- (int)enableVideoFrameObserver:(BOOL)enable position:(DingRtcVideoObservePosition)position;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 停止/恢复本地音频数据发送。
 * @param mute  
 *          - YES表示静音本地音频，发送静音帧。
 *          - NO表示取消静音。
 * @param mode  静音模式，静音模式分三种，详见{@link DingRtcMuteLocalAudioMode}, 默认模式为静音本地麦克风采集音频和所有外部输入音频。
 * @return
 * - 0：成功；
 * - 非0：失败。
 * @note 静音是指音频流发送静音帧,采集和编码模块仍然在工作。
 */
- (int)muteLocalMic:(BOOL)mute mode:(DingRtcMuteLocalAudioMode)mode;

/**
 * @ingroup OC_DingRtcEnginePub
 *
 * @since 3.0
 * @brief 是否推送音频流。
 * @param enabled 
 * - 是否开启/关闭本地音频流推流。
 * - YES：开启音频流推送。
 * - NO：关闭音频流推送。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 *  - DingRtcErrInner：SDK内部状态错误，需检查是否创建SDK实例成功。
 *
 * @note
 * - SDK默认设置推送音频流，加入频道前调用此接口设置推送音频流，将在加入频道成功时生效，加入频道成功后可以随时调用此接口开启/关闭音频流推送。
 * - 本地音频流内容默认为麦克风采集音频，如需自定义输入音频推送内容，可参考接口{@link DingRtcEngine::setExternalAudioSource:withSampleRate:channelsPerFrame:}使用外部音频输入替换麦克风采集。
 * - 本地音频推流结果发生变化时，SDK会触发{@link DingRtcEngineDelegate:onAudioPublishStateChanged:newState:elapseSinceLastState:channel:}回调通知音频推流最新状态。
 */
- (int)publishLocalAudioStream:(BOOL)enabled;

/** @} */

#pragma mark - "操作来自远端的媒体"
/**
 * @name 操作来自远端的媒体
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 设置远端视频显示视图。
 * @param canvas 本地视频显示属性, 详细见{@link DingRtcVideoCanvas}。
 * @param uid User ID，从App server分配的唯一标示符。
 * @param track  只支持{@link DingRtcVideoTrack#DingRtcVideoTrackCamera}和{@link DingRtcVideoTrack#DingRtcVideoTrackScreen}。
 * @return
 * - 0 ：成功；
 * - < 0 ：失败。
 * @note 支持joinChannel之前和之后切换窗口。如果canvas或者{@link DingRtcVideoCanvas#view}为nil，则停止显示。
 *       如果需要重新设置{@link DingRtcVideoCanvas#renderMode}，请保持canvas中其他属性不变，仅修改
 *       {@link DingRtcVideoCanvas#renderMode}。
 *       如果需要重新设置{@link DingRtcVideoCanvas#mirrorMode}，请保持canvas中其他属性不变，仅修改
 *       {@link DingRtcVideoCanvas#mirrorMode}。
 */
- (int)setRemoteViewConfig:(DingRtcVideoCanvas *_Nullable)canvas uid:(NSString *_Nonnull)uid forTrack:(DingRtcVideoTrack)track;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 更新本地或远端视频视图属性。
 * @param canvas 视频显示属性, 详细见{@link DingRtcVideoCanvas}。
 * @param uid User ID，从App server分配的唯一标示符。uid 为nil或者为""代表本地用户。
 * @param track  只支持{@link DingRtcVideoTrack#DingRtcVideoTrackCamera}和{@link DingRtcVideoTrack#DingRtcVideoTrackScreen}。
 * @return
 * - 0 ：成功；
 * - < 0 ：失败。
 * @note 支持joinChannel之前和之后设置。设置{@link DingRtcVideoCanvas#view}为需要更新的view，
 * 目前支持更新{@link DingRtcVideoCanvas#mirrorMode}，
 * {@link DingRtcVideoCanvas#renderMode}，
 * {@link DingRtcVideoCanvas#rotationMode}
 */
- (int)updateViewConfig:(DingRtcVideoCanvas *_Nonnull)canvas uid:(NSString *_Nullable)uid forTrack:(DingRtcVideoTrack)track;

/** @} */

#pragma mark - "用户管理"
/**
 * @name 用户管理
 * @{
 */

/**
 * @ingroup OC_DingRtcEnginePub
 *
 * @since 3.0
 * @brief 查询是否设置推送视频流。
 * @return
 * - YES：已设置推送；
 * - NO：未设置推送。
 * @note
 *  - SDK默认设置推送视频流，如果需要取消推送视频流，可以通过{@link  DingRtcEngine::publishLocalVideoStream:}方法设置为不推送。
*/
- (BOOL)isLocalVideoStreamPublished;

/**
 * @ingroup OC_DingRtcEnginePub
 *
 * @since 3.0
 * @brief 查询是否设置推送音频流。
 * @return
 * - YES：已设置推送；
 * - NO：未设置推送。
 * @note
 *  - SDK默认设置推送音频流，如果需要取消推送音频流，可以通过{@link DingRtcEngine::publishLocalAudioStream:}方法设置为不推送。
*/
- (BOOL)isLocalAudioStreamPublished;

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @remark ios
 * @brief 启动屏幕分享  (仅适用于 iOS 系统)。
 * @param appGroup 插件和宿主app约定使用相同的appGroup。
 * @param mode 屏幕共享模式。
 * @return
 * - 0：成功，
 * - 其他：失败。
 */
- (int)startScreenShare:(NSString * _Nullable)appGroup
                   mode:(DingRtcScreenShareMode)mode;
#endif

/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @brief 停止屏幕分享。
 * @return
 * - 0：成功；
 * - 其他：失败。
 */
- (int)stopScreenShare;

/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @brief 查询是否设置推送屏幕分享。
 * @return
 *  - YES：已设置推送；
 *  - NO：未设置推送。
*/
- (BOOL)isScreenSharePublished;

#if TARGET_OS_MAC && !TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @remark mac
 * @brief 根据桌面id进行屏幕分享 (仅适用于 Mac 系统)。
 * @param desktopId 桌面id(可通过GetScreenShareSourceInfo接口获取)。
 * @param config 屏幕分享配置，可指定分享区域，详见 {@link DingRtcScreenShareConfig}。
 * @return
 *  - 0：设置成功，
 *  - <0：设置失败，返回错误码。
 */
- (int)startScreenShareWithDesktopId:(NSString * _Nonnull)desktopId config:(DingRtcScreenShareConfig * _Nonnull)config;

/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @remark mac
 * @brief 根据窗口id进行屏幕分享 (仅适用于 Mac 系统)。
 * @param windowId 窗口id(可通过GetScreenShareSourceInfo接口获取)。
 * @param config 屏幕分享配置（暂不支持指定区域分享）。
 * @return
 *  - 0：设置成功，
 *  - <0：设置失败，返回错误码。
 * @note 根据窗口id进行屏幕分享暂不支持指定区域分享，既 config 无效果。
 */
- (int)startScreenShareWithWindowId:(NSString * _Nonnull)windowId config:(DingRtcScreenShareConfig * _Nonnull)config;

/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.4.0
 * @remark mac
 * @brief 更新当前正在进行中的屏幕分享的配置。
 * @param config 屏幕分享配置，详见 {@link DingRtcScreenShareConfig}。
 * @return
 *  - 0: 设置成功
 *  - <0: 失败。
 * @note 正在屏幕分享中才有效。桌面分享可进行更新，窗口分享暂不支持指定区域分享，也就不支持更新。
*/
- (int)updateScreenShareConfig:(DingRtcScreenShareConfig * _Nonnull)config;

/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @remark mac
 * @brief 获取屏幕分享源信息 (仅适用于 Mac 系统)。
 * @param type 屏幕分享类型，详见 {@link DingRtcScreenShareType}。
 * @return 返回屏幕共享源列表。
 */
- (NSArray<DingRtcScreenSourceInfo *> * _Nullable)getScreenShareSourceInfoWithType:(DingRtcScreenShareType)type;
#endif

/**
 * @ingroup OC_DingRtcEngineSub
 *
 * @since 3.0
 * @brief 设置订阅视频流分辨率规格。
 * @param uid  用户ID，从App server分配的唯一标示符。
 * @param streamType 视频流的分辨率规格，详见 {@link DingRtcVideoStreamType}。
 * @return
 * - 0：设置成功；
 * - 非0：设置失败。
 */
- (int)setRemoteVideoStreamType:(NSString *_Nonnull)uid type:(DingRtcVideoStreamType)streamType;

/**
 * @ingroup OC_DingRtcEngineSub
 *
 * @since 3.0
 * @brief 设置默认订阅的视频流分辨率规格；如果不设置，则默认订阅最大的分辨率规格。
 * @param streamType 视频流的分辨率规格，详见 {@link DingRtcVideoStreamType}。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 */
- (int)setRemoteDefaultVideoStreamType:(DingRtcVideoStreamType)streamType;

/**
 * @ingroup OC_DingRtcEngineSub
 *
 * @since 3.0
 * @brief 停止/恢复订阅所有远端音频流，包含后入会成员的订阅。
 * @param sub 
 * - 是否订阅所有远端音频流。
 * - YES：订阅所有用户的音频流，后入会成员会自动订阅其音频。
 * - NO：停止订阅所有用户的音频流，后入会成员也不会自动订阅其音频。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 *
 * @note
 * - 入会前入会后均可以调用。
 */
- (int)subscribeAllRemoteAudioStreams:(BOOL)sub;

/**
 * @ingroup OC_DingRtcEngineSub
 *
 * @since 3.0
 * @brief 停止/恢复订阅所有远端视频流，包含后入会成员的订阅。
 * @param sub 
 * - 是否订阅所有远端视频流。
 * - YES：订阅所有用户的视频流，后入会成员会自动订阅其视频。
 * - NO：停止订阅所有用户的视频流，后入会成员也不会自动订阅其视频。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 *
 * @note
 * - 入会前入会后均可以调用。
 * - 如果设置了 subscribeAllRemoteVideoStreams:NO，会取消订阅已订阅的远端视频流，同时后面再入会的人不会进行自动订阅；但是依然可以通过 {@link subscribeRemoteVideoStream:track:sub:} 来主动订阅某个人的远端视频流。
 */
- (int)subscribeAllRemoteVideoStreams:(BOOL)sub;

/**
 * @ingroup OC_DingRtcEngineSub
 * 
 * @since 3.0
 * @brief 停止/恢复订阅远端用户的视频流。
 * @param uid 用户ID，从App server分配的唯一标示符。
 * @param track 视频流类型，详见 {@link DingRtcVideoTrack}。
 * @param sub 
 * - 是否订阅远端用户的视频流。
 * - YES：订阅指定用户的视频流。
 * - NO：停止订阅指定用户的视频流。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 *
 * @note
* - 如果设置了 subscribeAllRemoteVideoStreams:NO，会取消订阅已订阅的远端视频流，同时后面再入会的人不会进行自动订阅；但是依然可以通过 {@link subscribeRemoteVideoStream:track:sub:} 来主动订阅某个人的远端视频流。
 */
- (int)subscribeRemoteVideoStream:(NSString *_Nonnull)uid track:(DingRtcVideoTrack)track sub:(BOOL)sub;

/**
 * @ingroup OC_DingRtcEngineUserInfo
 *
 * @since 3.0
 * @brief 获取远端在线用户列表。
 * @return 当前频道内所有在线用户userid列表。
 * @note 若当前频道ChannelProfile设置为{@link DingRtcInteractivelive}模式，只有设置角色为{@link DingRtcClientRoleInteractive}的用户为在线用户，本方法不返回角色为{@link DingRtcClientRoleLive}的用户。
*/
- (NSArray<NSString *> *_Nullable)getOnlineRemoteUsers;

/**
 * @ingroup OC_DingRtcEngineUserInfo
 *
 * @since 3.0
 * @brief 查询用户是否在线。
 * @param uid 用户ID，从App server分配的唯一标示符。不可为NULL。
 * @return
 *  - YES，用户在线；
 *  - NO，用户不在线。
 * @note 根据特定的UserID，查询该用户是否在线。
 */
- (BOOL)isUserOnline:(NSString *_Nonnull)uid;

/**
 * @ingroup OC_DingRtcEngineOthers
 *
 * @since 3.0
 * @brief 获取当前网络链接状态。
 * @return 当前链接状态, 参考{@link DingRtcConnectionStatus}。
 * @note 网络链接状态发生变化时，SDK会通过{@link onConnectionStatusChanged}回调通知。
 * - 入会前调用此接口，均返回状态DingRtcConnectionStatusDisconnected。
*/
- (DingRtcConnectionStatus)getCurrentConnectionStatus;

/** @} */

#pragma mark - "设备管理"
/**
 * @name 设备管理
 * @{
 */

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark ios
 * @brief 设置音频输出为听筒还是扬声器 (仅适用于 iOS 系统)。
 * @param enable YES:扬声器模式(默认扬声器)；NO:听筒模式。
 * @return
 * - 0：成功；
 * - <0：失败。
 */
- (int)enableSpeakerphone:(BOOL)enable;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark ios
 * @brief 获取当前音频输出为听筒还是扬声器 (仅适用于 iOS 系统)。
 * @return
 * - YES：扬声器模式；
 * - NO：听筒模式。
 */
- (BOOL)isSpeakerphoneEnabled;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4.2
 * @remark ios
 * @brief 设置SDK对AVAudioSession的控制权限
 * @param restriction 控制权限参数, 详见{@link DingRtcAudioSessionOperationRestriction}。
 * @return
 * - 0: 成功
 * - 非0: 失败
 */
- (int)setAudioSessionOperationRestriction:(DingRtcAudioSessionOperationRestriction)restriction;

#endif

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark ios
 * @brief 设置音频profile。
 * @param audio_profile 音频采集/编码模式参数, 详见{@link DingRtcAudioProfile}。
 * @param audio_scene 音频场景模式参数,详见{@link DingRtcAudioScenario}。
 * @return
 * - 0：成功；
 * - 其他：失败。
 * @note 只支持joinChannel前设置有效。
 */
- (int)setAudioProfile:(DingRtcAudioProfile)audio_profile audio_scene:(DingRtcAudioScenario)audio_scene;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 开始测试音频采集设备
 * @return 0表示Success 非0表示Failure
 * @note 请在joinChannel之前调用
 */
- (int)startRecordingDeviceTest;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 停止测试音频采集设备
 * @return 0表示Success 非0表示Failure
 * @note 请在joinChannel之前调用
 */
- (int)stopRecordingDeviceTest;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 开始测试音频播放设备
 * @param filePath 音频文件路径
 * @return 0表示Success 非0表示Failure
 * @note 请在joinChannel之前调用
 */
- (int)startPlayoutDeviceTest:(NSString *_Nonnull)filePath;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 停止测试音频播放设备
 * @return 0表示Success 非0表示Failure
 * @note 请在joinChannel之前调用
 */
- (int)stopPlayoutDeviceTest;

/**
 * @since 3.4.2
 * @brief 设置算法或者其他私有参数。
 * @param params 用于接收来自客户设置的参数，通过JSON配置SDK的特别功能。
 * @return 
 * - 0：成功；
 * - 其他：失败。
*/
- (int)setParameters:(NSString *_Nonnull)params;

#if TARGET_OS_IPHONE

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark ios
 * @brief 设置camera采集偏好 (仅适用于 iOS 系统)。
 * @param config
 *        - cameraDirection：设置采集方向，前置 或 后置。
 * @return 0为成功；非0失败。
 * @note 必须在打开摄像头之前设置，如{@link DingRtcEngine::startPreview}, {@link DingRtcEngine::joinChannel:name:onResultWithUserId:}之前设置。
*/
- (int)setCameraCapturerConfiguration:(DingRtcCameraCapturerConfiguration* _Nonnull )config;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark ios
 * @brief 切换前后摄像头 (仅适用于 iOS 系统)。
 * @return
 * - 0：表示Success；
 * - 非0：表示Failure。
 */
- (int)switchCamera;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 设置摄像头缩放。
 * @param zoom   缩放级别，范围从 1 到 camera支持的最大 zoom值。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
 - (int)setCameraZoom:(float)zoom;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 设置摄像头闪光灯开关。
 * @param flash  是否打开闪光灯。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setCameraFlash:(BOOL)flash;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 摄像头是否支持手动聚焦。
 * @return 
 * - YES：支持；
 * - NO：不支持。
 */
- (BOOL)isCameraFocusPointSupported;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 设置摄像头手动聚焦点。
 * @param point   聚焦点坐标，（相对坐标， 取值范围0~1）。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setCameraFocusPoint:(CGPoint)point;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 摄像头是否支持设置曝光点。
 * @return 
 * - YES：支持；
 * - NO：不支持。
 */
- (BOOL)isCameraExposurePointSupported;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 设置摄像头曝光点。
 * @param point  曝光点，（相对坐标， 取值范围0~1）。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setCameraExposurePoint:(CGPoint)point;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark ios
 * @brief 设置设备方向 (仅适用于 iOS 系统)。
 * @param orientation 设备方向。
 */
- (void)setDeviceOrientationMode:(DingRtcOrientationMode)orientation; 

#endif

#if TARGET_OS_MAC && !TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取系统中的录音设备列表 (仅适用于 Mac 系统)。
 */
- (NSArray<DingRtcDeviceInfo *> *_Nullable)getAudioCaptures;

/**
 * @brief 获取使用的录音设备ID (仅适用于 Mac 系统)。
 */
- (NSString *_Nullable)getCurrentAudioCaptureID;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 选择录音设备(ID) (仅适用于 Mac 系统)。
 * @param captureID  音频采集设备Id。
 */
- (void)setCurrentAudioCaptureWithID:(NSString *_Nonnull)captureID;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取系统中的扬声器列表 (仅适用于 Mac 系统)。
 */
- (NSArray<DingRtcDeviceInfo *> *_Nullable)getAudioRenderers;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取当前使用的扬声器ID (仅适用于 Mac 系统)。
 */
- (NSString *_Nullable)getCurrentAudioRendererID;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 选择扬声器(ID) (仅适用于 Mac 系统)。
 * @param rendererID  音频播放设备id。
 */
- (void)setCurrentAudioRendererWithID:(NSString *_Nonnull)rendererID;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 获取摄像头列表 (仅适用于 Mac 系统)。
 * @return 找到的系统中摄像头名称列表。
 */
- (NSArray<DingRtcDeviceInfo *> *_Nullable)getCameraList;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 获取摄像头列表 (仅适用于 Mac 系统)。
 * @return 回调方式找到的系统中摄像头名称列表。
 */
- (int)getCameraList:(void (^)(NSArray<DingRtcDeviceInfo *> *_Nullable))onResult;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 获取当前使用的摄像头名称 (仅适用于 Mac 系统)。
 */
- (NSString *_Nullable)getCurrentCamera;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 获取当前使用的摄像头ID (仅适用于 Mac 系统)。
 */
- (NSString *_Nullable)getCurrentCameraID;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 选择摄像头(Name) (仅适用于 Mac 系统)。
 * @param camera   摄像头名称。
 */
- (void)setCurrentCamera:(NSString *_Nonnull)camera;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 选择摄像头(ID) (仅适用于 Mac 系统)。
 * @param cameraID   摄像头ID。
 */
- (void)setCurrentCameraWithID:(NSString *_Nonnull)cameraID;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 设置音频采集设备音量 (仅适用于 Mac 系统)。
 * @param volume 音量范围 [0, 100]。
 * @return 0表示Success；非0表示Failure。
 */
- (int)setRecordingDeviceVolume:(NSInteger)volume;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取音频采集设备音量 (仅适用于 Mac 系统)。
 * @return 音频采集设备音量，音量范围 [0, 100]，返回-1表示失败。
 */
- (int)getRecordingDeviceVolume;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 设置音频播放设备音量 (仅适用于 Mac 系统)。
 * @param volume 音量范围 [0, 100]
 * @return 0表示Success；非0表示Failure。
 */
- (int)setPlayoutDeviceVolume:(NSInteger)volume;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取音频播放设备音量 (仅适用于 Mac 系统)。
 * @return 音频播放设备音量，音量范围 [0, 100]，返回-1表示失败。
 */
- (int)getPlayoutDeviceVolume;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 静音音频采集设备 (仅适用于 Mac 系统)。
 * @param mute TRUE：设备设置为静音，FALSE：设备设置为不静音。
 * @return 0：方法调用成功；<0：方法调用失败。
 */
- (int)setRecordingDeviceMute:(BOOL)mute;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取音频采集设备静音状态 (仅适用于 Mac 系统)。
 * @return TRUE：已静音状态；FALSE： 非静音状态。
 */
- (BOOL)getRecordingDeviceMute;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 静音音频播放设备 (仅适用于 Mac 系统)。
 * @param mute TRUE：设备设置为静音，FALSE：设备设置为不静音。
 * @return 0：方法调用成功；<0：方法调用失败。
*/
- (int)setPlaybackDeviceMute:(BOOL)mute;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取音频播放设备静音状态 (仅适用于 Mac 系统)。
 * @return TRUE：已静音状态；FALSE：非静音状态。
*/
- (BOOL)getPlaybackDeviceMute;

#endif

/** @} */

#pragma mark - "其他"
/**
 * @name 其他
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineChannel
 *
 * @since 3.0
 * @brief 获取状态：当前是否在频道中。
 * @return
 *  - YES，表示当前已经加入了频道。
 *  - NO, 表示当前不在频道中。
 * @note 该方法可在加入频道前或加入频道后调用。
 */
- (BOOL)isInCall;

/**
 * @ingroup OC_DingRtcEngineUserInfo
 *
 * @since 3.0
 * @brief 查询远端用户状态。
 * @param uid 用户ID。从App server分配的唯一标示符。
 * @return dict App提供的容器，用于存放用户数据，dict类型为字典类型，通过{Key,Value}进行访问。
 *  - dict[userID] != @""  成功。
 *  - dict[userID] == @""  失败。
 * @note 通过 uid 查询，指定远端用户的状态信息，在参数dict中，以字典{key,value}的形式，返回用户的全部状态信息。
 * - Key信息：
 *  - userID：远端用户的ID。
 *  - isOnline：远端用户是否在线。
 *  - displayName：远端用户的名称。
 *  - hasAudio：远端用户是否推送了音频流。
 *  - hasCamera：远端用户是否推送了相机流。
 *  - hasScreenSharing：远端用户是否推送了屏幕流。
 *  - requestAudio：本端是否订阅了此远端用户的音频流。
 *  - requestCamera：本端是否订阅了此远端用户的相机流。此状态，系在成功订阅该用户摄像头流前的状态。
 *  - requestScreenSharing：本端是否订阅了此远端用户的屏幕流。此状态，系在成功订阅该用户屏幕共享前的状态。
 *  - preferCameraType：如果网络带宽允许，优先订阅的相机流的大小规格，选择范围：FHD, HD, SD, LD。
 *  - subScribedAudio：本端是否拉到了此远端用户的音频流。
 *  - subScribedCameraType：本端拉到了此远端用户的相机流的大小规格，可能的值：FHD, HD, SD, LD。 
 *  - subScribedScreenSharing：本端是否拉到了此远端用户的屏幕流。
 *  - hasCameraView：本端是否设置了此远端用户相机流的view。
 *  - hasScreenView：本端是否设置了此远端用户屏幕流的view。
 *  - isCameraMirror：本端是否设置了此远端用户的相机流为镜像。
 *  - isScreenMirror：本端是否设置了此远端用户的屏幕流为镜像。
 */
- (NSDictionary *_Nullable)getUserInfo:(NSString *_Nonnull)uid;

/** @} */

#pragma mark - "音视频裸数据输入输出"
/**
 * @name 音视频裸数据输入输出
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 设置音量回调频率和平滑系数。
 * @param interval 时间间隔，单位毫秒，最小值不得小于10ms, 建议设置300-500ms, <= 0表示不启用音量提示和说话人提示功能。
 * @param smooth 平滑系数，数值越大平滑程度越高，反之越低，实时性越好，建议设置3，范围[0, 9]。
 * @param reportVad 
 * - 说话人检测开关
 * - 1：开启。
 * - 0：关闭。
 * @return
 * - 0：成功；
 * - <0：失败。
 * @note 设置之后，音频音量通过{@link DingRtcEngineDelegate::onAudioVolumeIndication:}回调。
 */
- (int)enableAudioVolumeIndication:(NSInteger)interval smooth:(NSInteger)smooth reportVad:(NSInteger)reportVad;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 设置是否启用外部音频输入源。
 * @param enable 
 *          - YES：开启。
 *          - NO：关闭。
 * @param sampleRate 采样率 16k, 48k...
 * @param channelsPerFrame 声道数 1：单声道; 2：双声道。
 * @return
 * - >=0：成功；
 * - <0：失败。
 */
- (int)setExternalAudioSource:(BOOL)enable withSampleRate:(NSUInteger)sampleRate channelsPerFrame:(NSUInteger)channelsPerFrame;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 输入外部音频数据推流到远端。
 * @param data 音频数据采样，详见 {@link DingRtcAudioDataSample}。
 * @return
 * - <0：表示失败；
 * - 0：表示成功。
 * - 返回值为{@link DingRtcErrorCode::DingRtcErrAudioBufferFull}时，需要在间隔投递数据时间长度后再次重试投递。
 * @note 为了保证语音连续性，SDK内部会有buffer缓存送进来的音频数据，以每次送10ms数据为例，如果返回{@link DingRtcErrorCode::DingRtcErrAudioBufferFull}时，说明内部缓冲区已满，建议等待20ms后再重新送当前数据帧。
 */
- (int)pushExternalAudioFrame:(DingRtcAudioDataSample *_Nonnull)data;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 注册音频数据回调。
 * @private
 * @param observer 音频数据回调，delegate = nil 则停止回调数据。
 * @note 需要调用enableAudioFrameObserver开启具体DingRtcAudioSource的回调。
 * @return
 *  - 0：成功；
 *  - 非0：失败。
 */
- (int)registerAudioFrameObserver:(id<DingRtcAudioFrameDelegate> _Nullable)observer;

/**
 * @ingroup OC_DingRtcEngineAudio
 * @since 3.0
 * @brief 订阅音频数据。
 * @private
 * @param enable 是否允许数据回调。
 * @param position 音频裸数据源类型，详见 {@link DingRtcAudioObservePosition}。
 * @return 0：sucess。
 */
- (int)enableAudioFrameObserver:(BOOL)enable position:(DingRtcAudioObservePosition)position;

/**
 * @ingroup OC_DingRtcEngineAudio
 * @since 3.3
 * @brief 订阅音频数据。
 * @private
 * @param enable 是否允许数据回调。
 * @param position 音频裸数据源类型，详见 {@link DingRtcAudioObservePosition}。
 * @param config 音频回调数据格式及权限，详见 {@link DingRtcAudioObserveConfig}。
 * @return 0：sucess。
 */
- (int)enableAudioFrameObserver:(BOOL)enable position:(DingRtcAudioObservePosition)position config:(DingRtcAudioObserveConfig *_Nonnull)config;

/**
 * @ingroup OC_DingRtcEngineAudio
 * @since 3.0
 * @brief 订阅音频数据。
 * @private
 * @param enable 是否允许数据回调。
 * @param audioSource 音频裸数据源类型，详见 {@link DingRtcAudioSource}。
 * @deprecated 该方法已废弃，请使用 enableAudioFrameObserver:position: 代替。
 * @return 0：sucess。
 */
- (int)enableAudioFrameObserver:(BOOL)enable audioSource:(DingRtcAudioSource)audioSource __attribute__((deprecated));

/**
 * @ingroup OC_DingRtcEngineAudio
 * @since 3.0
 * @brief 设置是否启用外部音频渲染源
 * @param enable YES:开启; NO:关闭
 * @param sampleRate 采样率 16k, 48k...
 * @param channelsPerFrame 声道数 1:单声道; 2:双声道
 * @return
 * - >=0: 成功
 * - <0: 失败
 */
- (int)setExternalAudioRender:(BOOL)enable withSampleRate:(NSUInteger)sampleRate channelsPerFrame:(NSUInteger)channelsPerFrame;

/**
 * @ingroup OC_DingRtcEngineAudio
 * @since 3.0
 * @brief 输入外部音频数据渲染到本地
 * @param data 音频数据采样，详见 {@link DingRtcAudioDataSample}
 * @return
 * - <0: 表示失败;
 * - 0: 表示成功;
 * - 返回值为{@link DingRtcErrorCode::DingRtcErrAudioBufferFull}时，需要在间隔投递数据时间长度后再次重试投递
 * @note 为了保证语音连续性，sdk内部会有buffer缓存送进来的音频数据，以每次送10ms数据为例，如果返回{@link DingRtcErrorCode::DingRtcErrAudioBufferFull}时，说明内部缓冲区已满，建议等待20ms后再重新送当前数据帧；
 */
- (int)pushExternalAudioRenderFrame:(DingRtcAudioDataSample *_Nonnull)data;

/**
 * @ingroup OC_DingRtcEngineAudio
 * @since 3.1
 * @brief 开启自定义音频采集, 开始后, SDK内部不在操作音频采集设备
 * @param enable 是否开启
 * @note 需要您在进入房间前设置才能生效，暂不支持进入房间后再设置
 * @note 使用自定义采集时, 不能在使用外部音频输入源{@link setExternalAudioSource}相关功能
 * @return
 * - 0: 成功
 * - <0: 失败
 */
- (int)enableCustomAudioCapture:(BOOL)enable;

/**
 * @ingroup OC_DingRtcEngineAudio
 * @since 3.1
 * @brief 向SDK投送自己采集的音频数据
 * @param data 音频帧，详见 {@link DingRtcAudioDataSample}
 * @note 音频帧数据只支持16位PCM格式, 10ms帧长, 采样率支持48000/44100/32000/16000, 声道数支持单声道/双声道
 * @return
 * - 0: 成功
 * - <0: 失败
 */
- (int)sendCustomAudioCaptureFrame:(DingRtcAudioDataSample *_Nonnull)data;

/**
 * @ingroup OC_DingRtcEngineAudio
 * @since 3.1
 * @brief 开启自定义音频播放, 开始后, SDK内部不在操作音频播放设备
 * @param enable 是否开启
 * @note 需要您在进入房间前设置才能生效，暂不支持进入房间后再设置
 * @note 使用自定义播放时, 不能在使用外部音频渲染源{@link setExternalAudioRender}相关功能
 * @return
 * - 0: 成功
 * - <0: 失败
 */
- (int)enableCustomAudioRender:(BOOL)enable;

/**
 * @ingroup OC_DingRtcEngineAudio
 * @since 3.1
 * @brief 获取可播放的音频数据
 * @param data 音频帧，详见 {@link DingRtcAudioDataSample}
 * @note 音频帧buffer需要使用者申请释放, 音频帧数据只支持16位PCM格式, 10ms帧长, 采样率支持48000/44100/32000/16000, 声道数支持单声道/双声道
 * @return
 * - 0: 成功
 * - <0: 失败
 */
- (int)getCustomAudioRenderFrame:(DingRtcAudioDataSample *_Nonnull)data;

/** @} */


#pragma mark - "音乐伴奏混音"

/**
 * @name 音乐伴奏混音
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 创建伴奏混音实例。
 * @param unqiueId 伴奏文件id，用户分配。
 * @param filePath 伴奏文件路径。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
-(int)createAudioMixing:(NSInteger)unqiueId filePath:(NSString *_Nonnull)filePath;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 销毁伴奏混音实例。
 * @param unqiueId 伴奏文件id。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
-(int)destroyAudioMixing:(NSInteger)unqiueId;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 开始伴奏混音。
 * @param unqiueId 伴奏文件id。
 * @param config 伴奏文件配置，参考{@link DingRtcAudioMixingConfig}。
 * @return
 * - 0：成功；
 * - 非0：失败。
 * @note 可通过{@link onAudioMixingStateChanged:}监听播放状态。
 */
-(int)startAudioMixing:(NSInteger)unqiueId config:(DingRtcAudioMixingConfig *_Nonnull)config;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 停止伴奏混音。
 * @param unqiueId 伴奏文件id。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
-(int)stopAudioMixing:(NSInteger)unqiueId;

/**
 * @ingroup OC_DingRtcEngineAudio
 * 
 * @since 3.4
 * @brief 暂停伴奏混音。
 * @param unqiueId 伴奏文件id。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
-(int)pauseAudioMixing:(NSInteger)unqiueId;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 暂停后恢复伴奏混音。
 * @param unqiueId 伴奏文件id。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
-(int)resumeAudioMixing:(NSInteger)unqiueId;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 获取伴奏时长。
 * @param unqiueId 伴奏文件id。
 * @return 伴奏时长，单位ms。返回值为-1时，说明获取时长失败。
 */
-(int)getAudioMixingDuration:(NSInteger)unqiueId;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 获取当前伴奏播放时长。
 * @param unqiueId 伴奏文件id。
 * @return 当前伴奏播放时长，单位ms。返回值为-1时，说明获取时长失败。
 */
-(int)getAudioMixingCurrentPosition:(NSInteger)unqiueId;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 设置伴奏播放位置。
 * @param unqiueId 伴奏文件id。
 * @param position 设置伴奏播放位置，单位ms。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
-(int)setAudioMixingPosition:(NSInteger)unqiueId position:(NSInteger)position;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 设置伴奏音量。
 * @param unqiueId 伴奏文件id。
 * @param volume 伴奏音量，取值[0, 100]。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
-(int)setAudioMixingVolume:(NSInteger)unqiueId volume:(NSInteger)volume;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 获取伴奏音量。
 * @param unqiueId 伴奏文件id。
 * @return 伴奏音量， < 0 代表获取失败。
 */
-(int)getAudioMixingVolume:(NSInteger)unqiueId;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 设置伴奏推流音量。
 * @param unqiueId 伴奏文件id。
 * @param volume 伴奏推流音量，取值[0, 100]。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
-(int)setAudioMixingPublishVolume:(NSInteger)unqiueId volume:(NSInteger)volume;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 获取伴奏推流音量。
 * @param unqiueId 伴奏文件id。
 * @return 伴奏推流音量， < 0 代表获取失败。
 */
-(int)getAudioMixingPublishVolume:(NSInteger)unqiueId;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 设置伴奏播放音量。
 * @param unqiueId 伴奏文件id。
 * @param volume 伴奏播放音量，取值[0, 100]。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
-(int)setAudioMixingPlayoutVolume:(NSInteger)unqiueId volume:(NSInteger)volume;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 获取伴奏播放音量。
 * @param unqiueId 伴奏文件id。
 * @return 伴奏播放音量， < 0 代表获取失败。
 */
-(int)getAudioMixingPlayoutVolume:(NSInteger)unqiueId;

/** @} */


#pragma mark - "文件录制"
/**
 * @name 文件录制
 * @{
 */

#if TARGET_OS_MAC && !TARGET_OS_IPHONE

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 开始录制 (仅适用于 Mac 系统)
 * @details SDK提供了开始录制的功能
 * @param recordOptions 录制视频内容及布局, 详细定义见{@link DingRtcRecordOptions}
 * @param filePath 录制文件的本地保存路径
 * @param fileName 录制文件名
 * @return
 * - YES：成功
 * - NO：失败
 */
- (BOOL)startRecord:(DingRtcRecordOptions*_Nonnull)recordOptions filePath:(NSString*_Nullable)filePath fileName:(NSString*_Nullable)fileName;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 更新录制内容信息 (仅适用于 Mac 系统)。
 * @details SDK提供了更新录制内容信息的功能,用于实时更新录制模板内容设置
 * @param layout 录制视频内容及布局, 详细定义见{@link DingRtcRecordVideoLayout}
 * @return
 * - YES：成功，
 * - NO：失败。
 * @note updateRecordLayout更新录制内容信息应该在录制过程中调用
 */
- (BOOL)updateRecordLayout:(DingRtcRecordVideoLayout*_Nonnull)layout;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 暂停录制 (仅适用于 Mac 系统)。
 * @details SDK提供了暂停录制的功能。
 * @return
 * - YES：成功，
 * - NO：失败。
 */
- (BOOL)pauseRecord;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 重新开启录制 (仅适用于 Mac 系统)。
 * @details SDK提供了重新开始录制的功能。
 * @return
 * - YES：成功，
 * - NO：失败。
 */
- (BOOL)resumeRecord;

/**
 * @ingroup OC_DingRtcEngineVideo
 * @since 3.0
 * @brief 停止录制 (仅适用于 Mac 系统)
 * @details SDK提供了停止录制的功能
 */
- (void)stopRecord;

#endif

/** @} */

#pragma mark - "external video source"
/**
 * @name external video source
 * @{
 */
/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 设置外部视频源。
*/
- (int)setExternalVideoSource:(BOOL)enable track:(DingRtcVideoTrack)track;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 推送外部视频数据。
*/
- (int)pushExternalVideoFrame:(DingRtcVideoFrame*_Nonnull)frame track:(DingRtcVideoTrack)track;

/** @} */

#pragma mark - "视频画面截图"
/**
 * @name 视频画面截图
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 对指定用户的视频进行截图，生成一张 JPG 格式的图片，并保存至指定的路径。
 * @param userId 用户ID，userId 为nil或者为""代表本地用户。
 * @param type 视频流类型，只支持 {@link DingRtcVideoTrack::DingRtcVideoTrackCamera} 和 {@link DingRtcVideoTrack::DingRtcVideoTrackScreen}。
 * @param filePath 截图的本地保存路径，需精确到文件名及格式，请确保目录存在且可写。如 xx/yy/zz.jpg。
 * @return
 * - 0：仅代表调用该接口成功，是否获取到截图画面，详细见 {@link DingRtcEngineDelegate::onSnapshotComplete:videoTrack:image:success:}；
 * - < 0：失败。
 * @note 视频截图功能为异步接口, 通过监听 {@link DingRtcEngineDelegate::onSnapshotComplete:videoTrack:image:success:} 回调获取当前截图画面。
 */
- (int)snapshotVideo:(NSString*_Nullable)userId type:(DingRtcVideoTrack)type filePath:(NSString* _Nonnull)filePath;

/** @} */

#pragma mark - "媒体扩展信息"
/**
 * @name 媒体扩展信息
 * @{
 */

/**
 * @brief 发送媒体扩展信息
 * @details SDK提供了发送和接收媒体扩展信息的功能，接收端参考 {@link DingRtcEngineDelegate::onMediaExtensionMsgReceived:message:}，使用场景：
 * - 使用媒体扩展信息传递时间戳，计算端到端的网络延迟，或者跟自身其他业务做数据同步
 * - 使用媒体扩展信息传递位控制信息。目前可以传递8 Byte数据，即64位，每一位或几位可以表示控制信息，用于自身业务上的指令传输
 *
 * @param data 扩展信息内容, 长度限制为最大8字节
 * @param repeatCount 重复次数，代表消息冗余度，用于防止网络丢包导致的消息丢失
 * @return
 * - 0: 成功
 * - <0: 失败
 *      - ERR_INNER(-1): SDK内部错误，可能的情况为SDK未初始化或者SDK销毁后调用
 *
 * @note 使用媒体扩展信息时需要复用音视频数据通道，因此必须控制自定义消息的发送频率和消息数据长度，使用限制如下：
 * - 每秒最多发送30条消息
 * - 为了不影响媒体数据的传输质量，自定义消息体长度限制为8 Byte，可以用来传输时间戳，位控制信息等
 * - sendMediaExtensionMsg函数中repeatCount参数为自定义消息冗余度，若大于1，则会发送多次，防止网络丢包导致的消息丢失，此时房间里的其他人也会收到多次相同的消息，需要去重
 * - 发送的自定义消息，在旁路直播时，房间里的订阅者也一样会收到
 * - 目前H5端不支持发送和接收媒体扩展信息
 */
- (int)sendMediaExtensionMsg:(NSData *_Nonnull)data repeatCount:(int)repeatCount;

/** @} */

#pragma mark - "音频分组"
/**
 * @name 音频分组
 * @{
 */

/**
 * 关于音频分组概念
 * 音频分组，允许频道内的成员建立工作组(group)。User加入一个group后，可以向这个group发言，也可以收听这个group内的音频。
 * 音频分组不影响视频的 publish 和 subscribe，即使2个user不在同一个group，也可以看见对方。
 * 大厅(hall)：如果一个user当前不在任何一个group中，那么他就留在大厅中。大厅是一个特殊的 group，通过离开/加入一个group间接的达到加入/离开大厅的效果。
 * 如果user在大厅中，那么他可以对大厅中的人发言，也可以订阅大厅音频。
 * 权限约束：user无权对不在其中的group (含hall)发言，或者收听音频。
 * 音频订阅约束：user不可以同时订阅多个group (含hall)的音频，这个是为了应对一个user出现在多个group中，以及过量的多路音频（比如6路音频）混音后造成理解困难的问题。
 */

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.4
 * @brief 返回音频分组大厅Id，可以根据此 Id 判断一个group是不是大厅。
 * @return 音频分组大厅Id。
 */
+ (NSString *_Nonnull)getAudioGroupHallId;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 加入音频分组。如果分组当前不存在，则自动创建一个。
 * @param groupId 指定加入的音频分组，不可以是大厅(audioGroupHallId)。
 * @param usrData 可以额外提供一个字符串，业务层自行解释。长度不超过64字节。
 * @return
 *  -  如果返回0，结果将会通过{@link onAudioGroupJoinResult}通知app。
 *  -  如果返回<0，则调用失败，不会有通知消息onAudioGroupJoinResult。
 * @note usrData暂时未支持。
 */
- (int)joinAudioGroup:(NSString*_Nonnull)groupId usrData:(NSString* _Nullable)usrData;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 离开音频分组。
 * @param groupId 要离开的音频分组，不可以是大厅(audioGroupHallId)。
 * @return
 *  -  如果返回0，结果将会通过{@link OnAudioGroupLeaveResult}通知app。
 *  -  如果返回<0，则调用失败，不会有通知消息OnAudioGroupLeaveResult。
 * @note 如果离开了分组，那么没有权限再订阅该分组的音频，也没有权限向该分组mix音频流。
 */
- (int)leaveAudioGroup:(NSString*_Nonnull)groupId;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 解散音频分组，所有成员被迫离开该音频分组。
 * @param groupId 要解散的音频分组，不可以是大厅(audioGroupHallId)。
 * @return
 *  -   如果返回0，结果将会通过{@link OnAudioGroupDismissResult}通知app。
 *  -   如果返回<0，则调用失败，不会有通知消息OnAudioGroupDismissResult。
 */
- (int)dismissAudioGroup:(NSString*_Nonnull)groupId;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 本地音频合并进分组音频或者大厅音频中。
 * @param groupId 指定的音频分组，可以是大厅(audioGroupHallId)。
 * @param mix true表示合并，false表示不合并。
 * @return
 *  -   如果返回0，结果将会通过{@link OnAudioGroupMixResult}通知app。
 *  -   如果返回<0，则调用失败，不会有通知消息OnAudioGroupMixResult。
 * @note 前提条件：1，已经做了音频的publish; 2，权限要求：用户需要在目标分组或者大厅中。
 */
- (int)mixAudioToGroup:(NSString*_Nonnull)groupId mix:(BOOL)mix;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 切换订阅的音频流到指定的分组或者大厅音频。
 * @param groupId 指定的音频分组，可以是大厅(audioGroupHallId)。
 * @return
 *  -   如果返回0，结果将会通过{@link OnAudioSubscribeStateChanged}通知app。
 *  -   如果返回<0，则调用失败，不会有通知消息OnAudioSubscribeStateChanged。
 * @note 业务层监听 onAudioSubscribeStateChanged 通知，如果uid为""，检查newState == DingRtcStatsSubscribed，再查询当前订阅成功的音频类型（通过{@link getCurrentSubscribedAudio:channelAudio:  }），可判断是否切换成功。
 * @note 切换过程会有多次 onAudioSubscribeStateChanged 通知。
 */
- (int)switchSubscriptionToAudioGroup:(NSString *)groupId;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.4
 * @brief 查询当前订阅的音频的类型，可能是大厅音频，也可能是某个分组的音频。
 * @return 返回的是 groupId，如果 nil 或者 ""，表示无订阅，如果是{@link audioGroupHallId}，表示大厅音频，否则，返回分组音频对应的分组ID。
 */
- (NSString *_Nullable)getCurrentSubscribedAudio;

/** @} */

#pragma mark - "工具方法"

/**
 * @name 工具方法
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineOthers
 *
 * @since 3.1.0
 * @brief 将 I420 格式的视频帧进行角度的旋转。
 * @param width 视频帧原始宽度。
 * @param height 视频帧原始高度。
 * @param mode 视频帧的旋转角度，一般为 0，90，180，270。
 * @return
 * - 0：成功
 * - < 0：失败。
 */
+ (int)I420RotateWithSrcY:(const unsigned char* _Nonnull)srcY srcStrideY:(int)srcStrideY
                     srcU:(const unsigned char* _Nonnull)srcU srcStrideU:(int)srcStrideU
                     srcV:(const unsigned char* _Nonnull)srcV srcStrideV:(int)srcStrideV
                     dstY:(unsigned char* _Nonnull)dstY dstStrideY:(int)dstStrideY
                     dstU:(unsigned char* _Nonnull)dstU dstStrideU:(int)dstStrideU
                     dstV:(unsigned char* _Nonnull)dstV dstStrideV:(int)dstStrideV
                    width:(int)width height:(int)height mode:(int)mode;

/** @} */

@end
